//
//  GLTrack.h
//  Trips
//
//  Created by Jeffrey Early on 9/30/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>
#import <GLAtlasKit/GLAtlasKit.h>

@class GLTrackSegment, GLEvent, GLTrackGeometry, GLSimplePointStyle;
@interface GLTrack : NSManagedObject <GLSpatialObject> {
	id objectOwner;
	GLTrackGeometry *geometry;
	GLSimplePointStyle *style;
}

/************************************************/
/*		Basic Attributes						*/
/************************************************/

#pragma mark -
#pragma mark Basic Attributes
#pragma mark

// If a cadence sensor was present, this reports the cadence over the entire track duration. (RPM)
@property(copy) NSNumber * cadence;

// If a heart rate monitor was present, this reports an estimate of the total calories burned. (kCal)
@property(copy) NSNumber * calories;

// Total distance. (meters)
@property(copy) NSNumber * distance;

// Time interval between the start time (the first point) and the last point. (seconds)
@property(copy) NSNumber * duration;

// "GPS comment for track." -- GPX1.1 specification --
@property(copy) NSString * gpsComment;

// Users chosen name for the track.
@property(copy) NSString * name;

// User notes. To be used as "desc" for the GPX1.1 specification
@property(copy) NSString * notes;

// "Type (classification) of track." -- GPX1.1 specification -- (Note: stored in GLTrackAdditions table).
@property(copy) NSString * type;

/************************************************/
/*		Calculated Attributes					*/
/************************************************/

#pragma mark -
#pragma mark Calculated Attributes
#pragma mark

// These attributes are a depended on the points associated with this track's segments. However, because querying
// the points database is an expensive operation, we compute and store these values whenever points are
// added or removed. -recalculateAttributes is automatically called when new segments are added or removed.
//
// In addition to computing the basic attributes, this function is also responsible for building the geometry cache.
- (void) recalculateAttributes;

@property(retain) NSNumber * numberOfTrackPoints;
@property(retain) NSDate * startDate;
@property(retain) NSDate * endDate;

/************************************************/
/*		Location Metadata						*/
/************************************************/

#pragma mark -
#pragma mark Location Metadata
#pragma mark

// Any of these may be 'nil' depending on a successful query to the AtlasKit
//@property(retain) GLCity * city;
//@property(retain) GLContinent * continent;
//@property(retain) GLCountry * country;
//@property(retain) GLCounty * county;
//@property(retain) GLState * state;

/************************************************/
/*		Track Segments							*/
/************************************************/

#pragma mark -
#pragma mark Track Segments
#pragma mark

@property(strong) NSArray *segments;
@property(readonly) NSUInteger countOfSegments;
- (GLTrackSegment *) objectInSegmentsAtIndex: (NSUInteger) index;
- (void) insertObject: (GLTrackSegment *) segment inSegmentsAtIndex: (NSUInteger) index;
- (void) removeObjectFromSegmentsAtIndex: (NSUInteger) index;
- (void) addSegment: (GLTrackSegment *) segment;
- (void) removeSegment: (GLTrackSegment *) segment;

/************************************************/
/*		Events									*/
/************************************************/

#pragma mark -
#pragma mark Events
#pragma mark

// Each track belongs to exactly one event
@property(retain) GLEvent * event;

// The order in which this this element was imported into its event.
@property NSInteger order;

// Tracks may belong to zero or more groups
@property(retain) NSSet* manualGroups;

// Is this even necessary? It should be entirely encapsulated in the geometry, no?
@property(readonly) GLRect bounds;

/************************************************/
/*		GLSpatialObject Protocol				*/
/************************************************/

#pragma mark -
#pragma mark GLSpatialObject Protocol
#pragma mark

@property(nonatomic, strong) id objectOwner;
@property(readwrite, strong, nonatomic) GLTrackGeometry *geometry;
@property(nonatomic, strong) GLSimplePointStyle *style;

/************************************************/
/*		Cache Locations                         */
/************************************************/

#pragma mark -
#pragma mark Cache Locations
#pragma mark

// Return the cached WKB data for the track, or nil if it doesn't exist.
- (NSData *) cachedWKBData;
- (NSData *) cachedWKBDataForSRID: (NSInteger) srid;

// Returns a unique path for a track with SRID=4326 -- regardless of whether something is there
- (NSString *) unprojectedGeometryCachePath;

// Returns a unique path for a given track and given SRID  -- regardless of whether something is there.
- (NSString *) geometryCachePathForSRID: (NSInteger) srid;

// Returns an array containing paths to all of the caches (all projections) associated with this geometry.
- (NSArray *) existingGeometryCachePaths;

// All existing cache paths, except for SRID 4326;
- (NSArray *) existingProjectedGeometryCachePaths;

- (void) clearExistingGeometryCachePaths;


@end

